import logging
import logging.handlers

import config.config as config

class Logger(object):
    def __init__(self, filename, fmt='%(asctime)s %(lineno)d %(levelname)s %(message)s'):
        self.filename = filename
        self.format_str = logging.Formatter(fmt)  # 设置日志格式

        self.logger = logging.getLogger(filename)
        self.logger.setLevel(logging.DEBUG)  # 设置日志级别

    def consoleHandler(self, console_level):
        sh = logging.StreamHandler()  # 往屏幕上输出
        sh.setFormatter(self.format_str)  # 设置屏幕上显示的格式
        sh.setLevel(console_level)
        self.logger.addHandler(sh)  # 把对象加到logger里

    def fileHandler(self, file_level, file_path):
        th = logging.handlers.RotatingFileHandler(file_path, 'a', 1024*2014, 10)
        th.setFormatter(self.format_str)  # 设置文件里写入的格式
        th.setLevel(file_level)
        self.logger.addHandler(th)

    def debug(self, info):
        self.logger.debug(info)

    def info(self, info):
        print(info)
        self.logger.info(info)

    def warning(self, info):
        self.logger.warning(info)

    def error(self, info):
        self.logger.error(info)


# 配置logger整体
logger = Logger(filename=config.LOGGER_NAME)
logger.consoleHandler(logging.INFO)

# 导入数据错误输出到 ERRLOGGER_NAME 文件
errlogger = Logger(config.ERRLOGGER_NAME)
errlogger.fileHandler(logging.ERROR, config.ERRLOGGER_FILE)