#!/bin/env python3
# -*- coding: UTF-8 -*-

"""
Copyright (c) 2015 hinacloud.cn, Inc. All Rights Reserved
@author padme(jinsilan@hinacloud.cn)
@brief

支持各种导入
"""
import argparse
import pprint
import sys
import traceback

# 忽略证书校验
import ssl
import sys as _sys
ssl._create_default_https_context = ssl._create_unverified_context

import config.config as config

import hinaanalytics
import hinaformatter
from config.hina_logger import logger
from config.hina_logger import errlogger

class SAArgumentParser(argparse.ArgumentParser):
    '''支持从文件读取 文件可以包含注释空行'''

    def convert_arg_line_to_args(self, arg_line):
        strip_line = arg_line.strip()
        # 空行
        if not strip_line:
            return []
        # 注释
        if strip_line.startswith('#'):
            return []
        fields = strip_line.split(':')
        # 只有一个参数 比如--debug
        if len(fields) == 1:
            return ['--%s' % fields[0]]
        # 两个参数，注意合理的strip()
        first = fields[0].strip()
        second = ':'.join(fields[1:]).strip()
        return ['--%s' % first, second]
"""
    def _read_args_from_files(self, arg_strings):
        # expand arguments referencing files
        new_arg_strings = []
        for arg_string in arg_strings:

            # for regular arguments, just add them back into the list
            if not arg_string or arg_string[0] not in self.fromfile_prefix_chars:
                new_arg_strings.append(arg_string)

            # replace arguments referencing files with the file content
            else:
                try:
                    with open(arg_string[1:], encoding='utf-8') as args_file:
                        arg_strings = []
                        for arg_line in args_file.read().splitlines():
                            for arg in self.convert_arg_line_to_args(arg_line):
                                arg_strings.append(arg)
                        arg_strings = self._read_args_from_files(arg_strings)
                        new_arg_strings.extend(arg_strings)
                except OSError:
                    err = _sys.exc_info()[1]
                    self.error(str(err))

        # return the modified argument list
        return new_arg_strings
"""

def parse_args():
    """
    注意几个parser的继承关系
    parent_parser -- profile_parser -- csv_profile/nginx_profile/mysql_profile
                 |-- event_parser -- csv_event/nginx_event/mysql_event
                 |-- json
    """
    # 1. 定义parser
    parent_parser = SAArgumentParser(add_help=False)
    # 公用参数
    # 1.1. url/path/project
    consumer_group = parent_parser.add_mutually_exclusive_group(required=True)
    consumer_group.add_argument('--url', '-l',
                                help='和--output_file选一个必填,发送数据的url，比如http://localhost:8088/hinadt, ' \
                                     '如果是云版则类似http://abc.cloud.hinacloud.cn:8088/hinadt?token=xxx, ' \
                                     'token请联系我们获取. 注意这个参数和--output_file不能同时使用',
                                default=None)
    consumer_group.add_argument('--output_file', '-O',
                                help='和--url选一个必填, 输出的文件名，输出每行是一个符合格式的json。注意这个参数和--url不可同时使用',
                                default=None)
    parent_parser.add_argument('--project', '-j',
                               help='可选，指定的project名，默认是None',
                               default=None,
                               required=False)
    parent_parser.add_argument('--log_level', '-lv',
                               help='可选，指定日志输出最小等级，默认 DEBUG',
                               default='DEBUG',
                               required=False)

    # 1.2 断点续传
    parent_parser.add_argument('--skip_cnt', '-c',
                               type=int,
                               help='第一次运行请忽略，如果运行失败，需要跳过成功的那集行，这个就是指定跳过几行的',
                               default=0)
    parent_parser.add_argument('--quit_on_error', '-Q',
                               action='store_true',
                               help='如果选中，则出现一条错误日志就会退出',
                               default=False)
    parent_parser.add_argument('--debug', '-D',
                               action='store_true',
                               help='如果指定了就是使用debug模式，不会导入数据，只在stdout显示数据，参见(https://himanual.hinadt.com/docs/debug_mode.html)',
                               default=False)

    # 2. profile/event有不同的选项
    # 2.1 profile 需要指定distinct_id
    profile_parent_parser = SAArgumentParser(add_help=False, parents=[parent_parser])
    # 指定account_id
    profile_parent_parser.add_argument('--account_id_from', '-df',
                                     help='必填, 从哪个字段作为account_id，如果指定，则每条数据算作对应字段的用户的行为.')
    profile_parent_parser.add_argument('--anonymous_id_from', '-ai',
                                     help='必填, 从哪个字段作为anonymous_id，如果指定，则每条数据算作对应字段的用户的行为.')

    profile_parent_parser.add_argument('--is_login',
                                       help='可选参数, distinct_id是否是login id，默认不是.',
                                       default=False,
                                       action='store_true')

    # 2.2 event
    event_parent_parser = SAArgumentParser(add_help=False, parents=[parent_parser])
    # 指定account_id
    event_parent_parser.add_argument('--account_id_from', '-df',
                                     help='必填, 从哪个字段作为account_id，如果指定，则每条数据算作对应字段的用户的行为.')
    event_parent_parser.add_argument('--anonymous_id_from', '-ai',
                                     help='必填, 从哪个字段作为anonymous_id，如果指定，则每条数据算作对应字段的用户的行为.')

    # event_parent_parser.add_argument('--is_login',
    #                                  help='可选参数, distinct_id是否是login id，默认不是.',
    #                                  default=False,
    #                                  action='store_true')

    event_parent_parser.add_argument('--is_login',
                                     help='可选参数, distinct_id是否是login id，默认不是.',
                                     default=False)

    # event必须要么指定event_from要么指定event_default
    event_group = event_parent_parser.add_mutually_exclusive_group(required=True)
    event_group.add_argument('--event_from', '-ef',
                             help='和event_default选一个必填。哪个字段作为event名，如果指定，则每条数据的事件名为对应字段的值。')
    event_group.add_argument('--event_default', '-ed',
                             help='和event_from选一个必填。默认的event名，如果指定，则将所有数据都算作这个event的。')
    # timestammp必须要么指定timestamp_from 要么指定timestamp_default 要么都不指定
    timestamp_group = event_parent_parser.add_mutually_exclusive_group(required=False)
    timestamp_group.add_argument('--timestamp_from', '-tf',
                                 help='哪个字段作为time, 如果指定，则每条数据对应的时间为对应字段的值.')
    timestamp_group.add_argument('--timestamp_default', '-td',
                                 help='默认的timestamp, 如果指定，则将所有数据都算作这个时间的事件。')
    event_parent_parser.add_argument('--timestamp_format', '-fm',
                                     help='和timestamp_from一起使用，如果指定, 并timestamp_from对应的字段是个字符串，可以通过这种方式指定时间格式。' \
                                          '默认是%%Y-%%m-%%d %%H:%%M:%%S',
                                     default='%Y-%m-%d %H:%M:%S')

    # 2.3 item
    item_parent_parser = SAArgumentParser(add_help=False, parents=[parent_parser])
    item_parent_parser.add_argument('--item_type',
                                    help='必填，指定 item 的 item_type',
                                    required=True)
    item_parent_parser.add_argument('--item_id',
                                    help='必填，指定 item 的 item_id',
                                    required=True)

    # 2.4 signup
    signup_parent_parser = SAArgumentParser(add_help=False, parents=[parent_parser])
    # signup_parent_parser.add_argument('--login_id_from',
    #                                   help='必填，指定用户关联的登录 ID',
    #                                   required=True)
    signup_parent_parser.add_argument('--account_id_from', '-df',
                                     help='必填, 从哪个字段作为account_id，如果指定，则每条数据算作对应字段的用户的行为.')
    signup_parent_parser.add_argument('--anonymous_id_from',
                                      help='必填，指定用户关联的匿名 ID',
                                      required=True)

    # 3. 支持五种数据格式
    parser = SAArgumentParser(description='通用格式文件导入工具,版本号%s' % config.VERSION)
    subparsers = parser.add_subparsers(dest='subparser_name')
    # 3.1 csv/mysql/nginx/oracle 支持分别导入event和profile
    for format_name in ['csv', 'mysql', 'nginx', 'oracle']:
        formatter_class = hinaformatter.get_formatter(format_name.title())
        sub_parser = subparsers.add_parser(
            '%s_profile' % format_name,
            parents=[profile_parent_parser],
            help='%s, 导入profile' % formatter_class.__doc__.splitlines()[0],
            formatter_class=argparse.RawDescriptionHelpFormatter,
            description=formatter_class.__doc__,
            fromfile_prefix_chars='@')
        formatter_class.add_parser(sub_parser)
        sub_parser = subparsers.add_parser(
            '%s_event' % format_name,
            parents=[event_parent_parser],
            help='%s, 导入event' % formatter_class.__doc__.splitlines()[0],
            formatter_class=argparse.RawDescriptionHelpFormatter,
            description=formatter_class.__doc__,
            # 指定命令行文件分隔符
            fromfile_prefix_chars='@')
        formatter_class.add_parser(sub_parser)
        sub_parser = subparsers.add_parser(
            '%s_item' % format_name,
            parents=[item_parent_parser],
            help='%s, 导入item' % formatter_class.__doc__.splitlines()[0],
            formatter_class=argparse.RawDescriptionHelpFormatter,
            description=formatter_class.__doc__,
            fromfile_prefix_chars='@')
        formatter_class.add_parser(sub_parser)
        sub_parser = subparsers.add_parser(
            '%s_signup' % format_name,
            parents=[signup_parent_parser],
            help='%s, 导入用户关联' % formatter_class.__doc__.splitlines()[0],
            formatter_class=argparse.RawDescriptionHelpFormatter,
            description=formatter_class.__doc__,
            fromfile_prefix_chars='@')
        formatter_class.add_parser(sub_parser)

    # 3.2 json直接继承parent parser 不区分event/profile/item
    json_formatter_class = hinaformatter.get_formatter('Json')
    sub_parser = subparsers.add_parser(
        'json',
        parents=[parent_parser],
        help=json_formatter_class.__doc__.splitlines()[0],
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description=json_formatter_class.__doc__,
        fromfile_prefix_chars='@')
    json_formatter_class.add_parser(sub_parser)

    # 4. 解析
    args = parser.parse_args()
    if not args.subparser_name:
        parser.print_help()
        sys.exit(1)
    return args

#######
#
# main需要的
#
######
def main():

    args = parse_args()
    # 文件日志输出设置，单个文件最大1m 最多10个文件
    logger.fileHandler(args.log_level, file_path=config.LOG_FILE)

    logger.debug('args: %s' % pprint.pformat(vars(args)))

    formatter = hinaformatter.get_formatter(args.subparser_name.split('_')[0].title(), args)
    if args.output_file:
        logger.info('logging file to %s only' % args.output_file)
        consumer = hinaanalytics.LoggingConsumer(args.output_file)
    else:
        hinaformatter.check_url(args.url)
        if args.debug:
            logger.info('running debug mode')
            consumer = hinaanalytics.DebugConsumer(args.url, False, 10000)
        else:
            logger.info('sending msg to %s directly' % args.url)
            consumer = hinaanalytics.BatchConsumer(args.url)
    formatter.set_consumer(consumer)

    skip_lines_cnt_down = args.skip_cnt
    total = formatter.get_total_num()
    if total == 0:
        raise Exception('cannot find valid line!')
    counter = {'total_write': 0, 'total_read': 0, 'error': 0, 'skip': args.skip_cnt}
    error = False
    if total > 0:
        counter['total'] = total
        progress_interval = 10000 if total < 10000 * 20 else (total / 100)
    else:
        progress_interval = 10000
    for record in formatter.read_records():
        try:
            if counter['total_read'] % progress_interval == 0:
                if total > 0:
                    percent = counter['total_read'] * 100.0 / counter['total']
                    logger.info('progress %.2f%% %s' % (percent, counter))
                else:
                    logger.info('progress %s' % counter)
            counter['total_read'] += 1
            if skip_lines_cnt_down > 0:
                counter['total_write'] += 1
                skip_lines_cnt_down -= 1
                continue
            record = formatter.parse_record(record)
            # 不同的事件类型关联的send 方法不一样，在formatter 初始化的时候关联的 TODO
            formatter.send(record)
            counter['total_write'] += 1
        except Exception as e:
            counter['error'] += 1
            logger.warning('failed to parse line %d' % counter['total_read'])
            logger.warning(traceback.format_exc())
            logger.warning(e)
            errlogger.logger.error(record)
            if args.quit_on_error:
                error = True
                break

    formatter.close()

    if error:
        logger.error('failed to import, please fix it and run with[--skip_cnt %d] again!' %
                     counter['total_write'])
        return 1
    else:
        logger.info('counter = %s.' % counter)
        if args.debug:
            logger.info('--debug 参数时不会真正导入数据')
        else:
            logger.info('数据已发送到海纳云，请查看埋点管理，确认数据是否校验通过')
        return 0


if __name__ == '__main__':
    sys.exit(main())
